<?php
// wbb imports
require_once(WBB_DIR.'lib/data/thread/ThreadList.class.php');

/**
 * LatestActiveThreadList provides extended functions for displaying the list of the latest active threads.
 *
 * @author 	Nerd Almighty
 * @copyright	2009-2010 Bergwerk <http://bergwerk.forenberg.de>
 * @license	Creative Commons - Attribution-Noncommercial-Share Alike 3.0 Unported <http://creativecommons.org/licenses/by-nc-sa/3.0/>
 * @package	com.bergwerk.wbb.page.user.latestPostsAndThreads
 * @subpackage	data.thread
 * @category 	Burning Board
 */
class LatestActiveThreadList extends ThreadList {
	// parameters
	public $boardIDs = '';
	public $filterInvisibleThreads = true;
	public $daysPrune = 1000;
	
	// data
	public $boardIDsForDeletedThreads = '';
	public $boardIDsForDisabledThreads = '';
	public $maxLastPostTime = 0;
	
	// sql
	public $sqlConditionVisibility = '';
	public $sqlConditionLanguage = '';

	/**
	 * Creates a new LatestActiveThreadList object.
	 *
	 * @param	string		$boardIDs			a list of board ids
	 * @param	boolean		$filterInvisibleThreads		should be true, to exclude any (including disabled or deleted) threads, that the user is not allowed to see
	 * @param	integer		$daysPrune			exclude any threads older than $daysPrune days	
	 * @param	integer		$presortBoardIDs		shrink the range of possible board ids to $presortBoardIDs, to save performance; can't be used with any kind of offset, since it will only work properly for the latest X threads / posts
	*/
	public function __construct($boardIDs = '0', $filterInvisibleThreads = true, $daysPrune = 1000, $presortBoardIDs = 0) {
		$this->boardIDs = $boardIDs;
		$this->filterInvisibleThreads = $filterInvisibleThreads;
		$this->daysPrune = $daysPrune;
		
		if (!empty($this->boardIDs)) {
			$allBoardIDs = explode(',', $this->boardIDs);
			$boardIDsForDeletedThreads = array();
			$boardIDsForDisabledThreads = array();
			
			if ($filterInvisibleThreads) {
				foreach ($allBoardIDs as $boardID) {
					if (WCF::getUser()->getBoardModeratorPermission('canReadDeletedThread', $boardID)) $boardIDsForDeletedThreads[] = $boardID;
					if (WCF::getUser()->getBoardModeratorPermission('canEnableThread', $boardID)) $boardIDsForDisabledThreads[] = $boardID;				
				}
			}
			
			if ($presortBoardIDs && count($allBoardIDs) > $presortBoardIDs) {
				$reducedBoardIDs = array();
				
				// prepare sql conditions
				$sqlConditions = "(thread.boardID IN (" . implode(',', $allBoardIDs) . ") AND thread.isDisabled = 0 AND thread.isDeleted = 0)";
				if (count(WCF::getSession()->getVisibleLanguageIDArray()) && (BOARD_THREADS_ENABLE_LANGUAGE_FILTER_FOR_GUESTS == 1 || WCF::getUser()->userID != 0)) {
					$sqlConditions = "thread.languageID IN (".implode(',', WCF::getSession()->getVisibleLanguageIDArray()).") AND " . $sqlConditions;
				}
				
				// get relevant board ids
				$sql = "SELECT		last_post.boardID
					FROM 		wbb".WBB_N."_board_last_post last_post
					LEFT JOIN	wbb".WBB_N."_thread thread
					ON		(thread.threadID = last_post.threadID)
					WHERE		thread.movedThreadID = 0 AND ".$sqlConditions."
					ORDER BY	thread.lastPostTime DESC";
				$result = WCF::getDB()->sendQuery($sql, $presortBoardIDs);
				while ($row = WCF::getDB()->fetchArray($result)) {
					$boardID = $row['boardID'];
					$reducedBoardIDs[] = $boardID;
				}
				
				// replace old board ids
				$this->boardIDs = (count($reducedBoardIDs) ? implode(',', $reducedBoardIDs) : '0');
			}

			$this->boardIDsForDeletedThreads = (count($boardIDsForDeletedThreads) ? implode(',', $boardIDsForDeletedThreads) : '0');
			$this->boardIDsForDisabledThreads = (count($boardIDsForDisabledThreads) ? implode(',', $boardIDsForDisabledThreads) : '0');
		}
		
		parent::__construct();
	}
	
	/**
	 * @see ThreadList::initDefaultSQL()
	*/
	protected function initDefaultSQL() {
		parent::initDefaultSQL();
		
		if ($this->filterInvisibleThreads) $this->sqlConditions = "((thread.boardID IN (" . $this->boardIDs . ") AND thread.isDisabled = 0 AND thread.isDeleted = 0 AND thread.movedThreadID = 0)";
		else $this->sqlConditions = "((thread.boardID IN (" . $this->boardIDs . ") AND thread.movedThreadID = 0)";
				
		// thread visibility
		if (!empty($this->boardIDsForDeletedThreads)) $this->sqlConditionVisibility .= " OR (thread.boardID IN (" . $this->boardIDsForDeletedThreads . ") AND thread.isDeleted = 1 AND thread.isDisabled = 0 AND thread.movedThreadID = 0)";
		if (!empty($this->boardIDsForDisabledThreads)) $this->sqlConditionVisibility .= " OR (thread.boardID IN (" . $this->boardIDsForDisabledThreads . ") AND thread.isDeleted = 0 AND thread.isDisabled = 1 AND thread.movedThreadID = 0)";
		$this->sqlConditions .= $this->sqlConditionVisibility . ")";
		
		// thread language
		if (WCF::getSession()->getVisibleLanguages()) {
			$this->sqlConditionLanguage = " AND (thread.languageID IN (".WCF::getSession()->getVisibleLanguages()."))";
			$this->sqlConditions .= $this->sqlConditionLanguage;
		}
		
		// days prune
		if ($this->daysPrune != 1000) {
			$this->sqlConditions .= " AND (lastPostTime >= ".(TIME_NOW - ($this->daysPrune * 86400)).")";
		}
	}
	
	/**
	 * @see ThreadList::readThreads()
	*/
	public function readThreads() {
		parent::readThreads();
		
		// select lastPostTime as last modified parameter for spiders
		foreach ($this->threads as $key => $thread) {
			if ($thread->lastPostTime > $this->maxLastPostTime) $this->maxLastPostTime = $thread->lastPostTime;
		}
	}
}
?>